// change_hairy.cmd
      
// Some commands to move the chip in all three directions.
// For solder vertices, these commands interpolate between the
// chip and the pad by means of a parameter "dist", which is
// the ratio of the distance to the pad to the sum of the distances
// to the chip and the pad, i.e. dist = 0 on the pad and dist = 1
// on the chip.
// Note: It is important to check that these move commands do
// exactly what you want, especially on surfaces that you don't
// normally see, else the force calculations below will be wrong.
// Doing "show facets where color == white" or 
// "show facets where color != white" before testing the moves will
// let you see inside.

// First, calculation of the relative distance of each vertex from the
// tombstone and the pad.  The vertex attribute "dist" is 0 for vertices
// on the pad, and 1 for vertices on the chip.
define vertex attribute dist real
calc_dists := { 
    mc := cos(tilt*pi/180); ms := sin(tilt*pi/180);
    foreach vertex vv do 
    { if ( on_constraint 1 or on_constraint 2 or on_constraint 3
           or on_constraint 4 or on_constraint 5 or
           on_constraint 30 or on_constraint 31 or on_constraint 32
           or on_constraint 33 or on_constraint 34 or on_constraint 36 ) 
      then vv.dist := 1
      else if fixed then vv.dist := 0
      else
      { chipdist := maximum(0,(x-x0)*mc-(z-z0)*ms)
                + maximum(0,y-TWidth/2)
                + maximum(0,-y-TWidth/2)
                + maximum(0,(x-x0)*ms+(z-z0)*mc- THeight)
                + maximum(0,-((x-x0)*ms+(z-z0)*mc));
        vv.dist := z/(z+chipdist);
      }
    };
}
new_x0 := x0
change_x0 := { calc_dists;
               change_dx := new_x0 - x0; x0 := new_x0;
               set vertex x x+dist*change_dx;
               recalc;
             }

new_y0 := y0
change_y0 := { calc_dists;
               change_dy := new_y0 - y0; y0 := new_y0;
               set vertex y y+dist*change_dy;
               recalc;
             }
new_z0 := z0
change_z0 := { calc_dists;
               change_dz := new_z0 - z0; old_z0 := z0; z0 := new_z0;
               set vertex z z+dist*change_dz;
               recalc;
             }
new_tilt := tilt
change_tilt := { calc_dists;
    change_dt := new_tilt - tilt; tilt := new_tilt;
    st := sin(change_dt*pi/180); ct := cos(change_dt*pi/180);
    foreach vertex vv do
    { dx := x - x0; dz := z - z0;
      set vv x x0+((1-dist)*dx + dist*(ct*dx+st*dz));
      set vv z z0+((1-dist)*dz + dist*(-st*dx+ct*dz));
    };
    recalc;
  }

