// force.cmd

// Force calculations.  Using the principle of virtual work, the force
// on the chip in each direction is calculated as the negative derivative of
// the energy with respect to that direction.  Central differences are
// used for greater accuracy.  It is assumed that forces are calculated
// at an equilibrium, so the surface does not have to be re-evolved
// after each perturbation.  Instead, the pressure can be used to 
// compensate for the volume change, since pressure is the Lagrange
// multiplier for the volume constraint.  The surface is left in its
// original position after each calculation.
// Note: To check these force calculations, you can do a perturbation
// by hand with evolution.

// This file should be read in after change_simple.cmd or change_hairy.cmd

calc_xf := { f_dx := PW_x/1000;  // small shift
             new_x0 := x0 + f_dx; change_x0; 
             energy_hi := total_energy - body[1].pressure*(body[1].volume-body[1].target);
             new_x0 := x0 - 2*f_dx; change_x0; 
             energy_lo := total_energy - body[1].pressure*(body[1].volume-body[1].target);
             xforce := -(energy_hi - energy_lo)/(2*f_dx);
             new_x0 := x0 + f_dx; change_x0; 
             printf "xforce: %17.15g\n",xforce;
           }

calc_yf := { f_dy := PW_y/1000;  // small shift
             new_y0 := y0 + f_dy; change_y0; 
             energy_hi := total_energy - body[1].pressure*(body[1].volume-body[1].target);
             new_y0 := y0 - 2*f_dy; change_y0; 
             energy_lo := total_energy - body[1].pressure*(body[1].volume-body[1].target);
             yforce := -(energy_hi - energy_lo)/(2*f_dy);
             new_y0 := y0 + f_dy; change_y0; 
             printf "yforce: %17.15g\n",yforce;
           }

calc_zf := { f_dz := THeight/1000;  // small shift
             new_z0 := z0 + f_dz; change_z0; 
             energy_hi := total_energy - body[1].pressure*(body[1].volume-body[1].target);
             new_z0 := z0 - 2*f_dz; change_z0; 
             energy_lo := total_energy - body[1].pressure*(body[1].volume-body[1].target);
             zforce := -(energy_hi - energy_lo)/(2*f_dz);
             new_z0 := z0 + f_dz; change_z0; 
             printf "zforce: %17.15g\n",zforce;
           }

//  Handy command to calculate all the forces
calc_all := { calc_xf; calc_yf; calc_zf }

